%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ROBUST.M

% This is a GAUSS program, written by
% Bruce E. Hansen
% Department of Economics
% University of Wisconsin
% bhansen@ssc.wisc.edu
% http:/www.ssc.wisc.edu/~bhansen

% This program estimates the spline transfer function model presented in
% "How Responsive are Private Transfers to Income?
% Evidence From a Laissez-Faire Economy"
% by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.

% The results of this program generates the estimates reported in Table 6.

% This program will take a long time to execute if one sets qn=0.  To speed up
% execution (with little cost in precision), set qn=500 or even qn=100.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trm = .02;            % Percentile of high-income households deleted  @
per_ = .10;           % Percentage to trim from quantiles for threshold search @
qn   = 0;             % Set qn=0  to search across all thresholds.
                      % Set qn to a positive integers to search
                      % across a fixed number of quantiles     @
noinc_=1;             % If set to 1, includes dummy for no income, else set to 0 @

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

out=fopen('robust.txt','wt');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model =1;
while model<=9
    smp=1;
    while smp<=2
        fprintf(out,'************************************************************\n\n\n');
        noinc_=1;
        if model==1
            fprintf(out,'International Transfer Recipients Excluded\n');
            if smp==1
                noinc_=1;
            else
                noinc_=0;
            end;
        elseif model==2
            fprintf(out,'Not Secondary Graduate\n');
        elseif model==3
            fprintf(out,'Secondary Graduate\n');
        elseif model==4
            fprintf(out,'Family Size 4 or Less\n');
        elseif model==5
            fprintf(out,'Family Size 5 or More\n');
            if smp==1
                noinc_=1;
            else
                noinc_=0;
            end;
        elseif model==6
            fprintf(out,'House Hold Head Aged 45 or Less\n');
            if smp==1
                noinc_=1;
            else
                noinc_=0;
            end;
        elseif model==7
            fprintf(out,'House Hold Head Over Aged 45\n');
            if smp==1
                noinc_=1;
            else
                noinc_=0;
            end;
        elseif model==8
            fprintf(out,'Gross Transfers Received\n');
        elseif model==9
            fprintf(out,'Gross Transfers Given (Gifts)\n');
        end;
        
        rname=['Region1_';
        'Region2_';
        'Region3_';
        'Region4_';
        'Region5_';
        'Region6_';
        'Region7_';
        'Region8_';
        'Region9_';
        'Region10';
        'Region11';
        'Region12';
        'Region13'];
    if smp==1;
        sample='Urban';
        rname=['Region0_';rname];
        knot=21840;
        load urban.txt;
        dat=urban;
    elseif smp==2;
        sample='Rural';
        knot=10793;
        load rural.txt;
        dat=rural;
    end;
    
    abroad=dat(:,1);
    domestic=dat(:,2);
    inkind=dat(:,3);
    gifts=dat(:,4);
    totinc=dat(:,5);
    retinc=dat(:,10);
    yname='Transfers';
    i = totinc+gifts-abroad-domestic-inkind-retinc;
    ii=sortrows(i,1);
    qi=ii(floor(length(dat(:,1))*(1-trm)));
    
    dd=(i<0)+(i>qi);
    if model==1;
        dd=dd+(1-dd).*(abroad>0);
    elseif model==2
        dd=dd+(1-dd).*(dat(:,14)+dat(:,15)+dat(:,16)>0);
    elseif model==3
        dd=dd+(1-dd).*(dat(:,12)+dat(:,13)>0);
    elseif model==4
        dd=dd+(1-dd).*(dat(:,23)>4);
    elseif model==5
        dd=dd+(1-dd).*(dat(:,23)<=4);
    elseif model==6
        dd=dd+(1-dd).*(dat(:,17)>45);
    elseif model==7
        dd=dd+(1-dd).*(dat(:,17)<=45);
    end;
    
    if model==1
        y=domestic+inkind-gifts;
    elseif model==8
        y=abroad+domestic+inkind;
    elseif model==9
        y=gifts;
    else
        y=abroad+domestic+inkind-gifts;
    end;
    
    tempdat=[dat,i,y];
    ind=0;
    for iii=1:length(dd)
        if dd(iii)==0
            if ind==0
                dat=tempdat(iii,:);
                ind=1;
            else
                dat=[dat;tempdat(iii,:)];
            end;
        end;
    end;
    n=length(dat(:,1));
    retinc=dat(:,10);
    i=dat(:,28);
    y=dat(:,29);
    
    dat(:,23)=dat(:,23)-dat(:,20)-dat(:,21)-dat(:,22);
    
    if model==2
        xx=[(retinc>0),retinc,dat(:,17)];
        xn='Has_Retr';
    elseif model==3
        xx=[(retinc>0),retinc,dat(:,17)];
        xn='Has_Retr';
    else
        xx=[(retinc>0),retinc,dat(:,12:17)];
    end;
    
    region=dat(:,27);
    r=unique(region);
    xreg=(region*ones(1,length(r))==(ones(length(region),1)*r'));
    xx=[xx,dat(:,18:26),xreg];
    mx=ones(n,1)*mean(xx);
    
    if noinc_==1
        i0=(i==0);
        xx=[i0,xx];
        xn='NoIncome';
    end;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
    % Linear Spline Model %
    xm=xx*inv(xx'*xx);
    e0=y-xm*(xx'*y);
    ee0=e0'*e0;
    q=sortrows(i,1);
    if qn==0
        q=unique(q(floor(per_*n):n-floor(per_*n)));
        qn=length(q(:,1));
    else
        q=q(floor((per_:(1-per_*2)/qn:(qn-1)*(1-per_*2)/qn+per_)*n))';
    end;
    
    sse=zeros(qn,1);
    j=1;
    while j<=qn
        qj=q(j);
        ic=i-qj;
        i1=(i<=qj);
        i2=1-i1;
        dx=[(ic.*i1),(ic.*i2)];
        xv=dx-xm*(xx'*dx);
        et=e0-xv*(e0'/xv')';
        sse(j)=et'*et;
        %if round(j/10)==(j/10)
        %    [j,qj,sse(j)/n]
        %end;
        j=j+1;
    end;
    [temp,jmin]=min(sse);
    knot=q(jmin);
    
    ic=i-knot;
    i1=(i<=knot);
    i2=1-i1;
    dx=[(ic.*i1),(ic.*i2)];
    x=[dx,xx];
    beta=(y'/x')';
    e=y-x*beta;
    xh=-beta(1).*i1-beta(2).*i2;
    x=[x,xh];
    tr1=dx*beta(1:2)+mx*beta(3+noinc_:length(beta(:,1)));
    
    xname=['_Slope_1';'_Slope_2'];
    theta=[beta;knot];
    
    sh=strcat(sample,' households');
    fprintf(out,'%s\n\n\n',sh);
    fprintf(out,'Observations:        %u\n',n);
    fprintf(out,'------------------------------------------------\n');
    ee=e'*e;
    sig=ee/(length(y(:,1))-length(x(1,:)));
    mi=inv(x'*x);
    se=sqrt(abs(diag(mi*sig)));
    seh=sqrt(abs(diag(mi*((x.*(e*ones(1,length(x(1,:)))))'*((x.*(e*ones(1,length(x(1,:)))))))*mi)));
    fprintf(out,'Variable    Estimate      St Error     Het St Er\n');
    fprintf(out,'----------------------------------\n');
    if noinc_==0
        fprintf(out,'_Slope_1   %f   %f   %f\n',theta(1),se(1),seh(1));
        fprintf(out,'_Slope_2   %f   %f   %f\n',theta(2),se(2),seh(2));
        fprintf(out,'Threshold  %f   %f   %f\n',theta(length(se(:,1))),se(length(se(:,1))),seh(length(se(:,1))));
    else
        fprintf(out,'_Slope_1   %f   %f   %f\n',theta(1),se(1),seh(1));
        fprintf(out,'_Slope_2   %f   %f   %f\n',theta(2),se(2),seh(2));
        fprintf(out,'NoIncome   %f   %f   %f\n',theta(3),se(3),seh(3));
        fprintf(out,'Threshold  %f   %f   %f\n',theta(length(se(:,1))),se(length(se(:,1))),seh(length(se(:,1))));
    end;
    fprintf(out,'\n\n\n');
     
    smp=smp+1;
  end;
   model=model+1;
end;
fclose(out);