%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GMM.M

% This is a Matlab program, written by
% Bruce E. Hansen
% Department of Economics
% University of Wisconsin
% bhansen@ssc.wisc.edu
% http:/www.ssc.wisc.edu/~bhansen

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
trm = .02;            % Percentile of high-income households deleted  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
out=fopen('gmm.txt','wt');
smp=1;
while smp<=2;  % Set to 1 if "urban" or 2 if "rural"  %
    rname=['Region1_';
        'Region2_';
        'Region3_';
        'Region4_';
        'Region5_';
        'Region6_';
        'Region7_';
        'Region8_';
        'Region9_';
        'Region10';
        'Region11';
        'Region12';
        'Region13'];
    if smp==1;
        sample='Urban';
        rname=['Region0_';rname];
        knot=21840;
        load urban.txt;
        dat=urban;
    elseif smp==2;
        sample='Rural';
        knot=10793;
        load rural.txt;
        dat=rural;
    end;
    
    abroad=dat(:,1);
    domestic=dat(:,2);
    inkind=dat(:,3);
    gifts=dat(:,4);
    totinc=dat(:,5);
    retinc=dat(:,10);
    yname='Transfers';
    
    y = abroad+domestic+inkind-gifts;
    i = totinc+gifts-abroad-domestic-inkind-retinc;
    
    ind=0;
    ttempdat=[dat,i,y];
    for iii=1:length(i);
        if i(iii)>=0
            if ind==0
                tempdat=ttempdat(iii,:);
                ind=1;
            else
                tempdat=[tempdat;ttempdat(iii,:)];
            end;
        end;
    end;
    dat=tempdat;
    i=sortrows(dat(:,28),1);
    n=floor(length(dat(:,1))*(1-trm));
    
    ind=0;
    for iii=1:length(dat(:,1))
        if dat(iii,28)<=i(n)
            if ind==0
                tempdat=dat(iii,:);
                ind=1;
            else
                tempdat=[tempdat;dat(iii,:)];
            end;
        end;
    end;
    dat=tempdat;
    
    retinc=dat(:,10);
    i=dat(:,28);
    y=dat(:,29);
    dat=dat(:,1:27);
    
    dat(:,23)=dat(:,23)-dat(:,20)-dat(:,21)-dat(:,22);
    xx=[(retinc>0),retinc,dat(:,12:17)];
    
    
    region=dat(:,27);
    r=unique(region);
    rr=length(r(:,1));
    xreg=(region*ones(1,length(r))==(ones(length(region),1)*r'));%???????%
    mx=ones(n,1)*mean(xx);
    
    n=length(dat(:,1));
    x1=[xx,dat(:,18:26),ones(n,1)];
    x2=[xx,dat(:,18:26),xreg];
    %xn=[xn;vn;rname]; %???????????%;
    
    % Reduced form for endogenous income @
    rs=xreg(:,1);
    ind=0;
    for iii=1:length(rs)
        if rs(iii)>0
            if ind==0
                ir=i(iii,:);
                xr=x1(iii,:);
                ind=1;
            else
                ir=[ir;i(iii,:)];
                xr=[xr;x1(iii,:)];
            end;
        end;
    end;
    ihat=xr*(ir'/xr')';
    j=2;
    while j<=rr
        rs=xreg(:,j);
        ind=0;
        for iii=1:length(rs)
            if rs(iii)>0
                if ind==0
                    ir=i(iii,:);
                    xr=x1(iii,:);
                    ind=1;
                else
                    ir=[ir;i(iii,:)];
                    xr=[xr;x1(iii,:)];
                end;
            end;
        end;
        ih=xr*(ir'/xr')';
        ihat=[ihat;ih];
        j=j+1;
    end;
    z=[ihat,ihat.^2,x2];  % instruments %
    
    fprintf(out,'************************************************************\n\n\n');
    sh=strcat(sample,' households');
    fprintf(out,'%s\n',sh);
    fprintf(out,'Dependent Variable:   %s\n',yname);
    fprintf(out,'Trimmed Top Quantile of Income:  %f\n',trm);
    fprintf(out,'Observations:                    %u\n',n);
    fprintf(out,'-------------------------------------------------\n');
    

    xname=['__Income';
               'Has_Retr';
               '__RetInc';
               '_Primary';
               'Some_Hgh';
               'Hign_Grd';
               'Some_Col';
               '_College';
               '_____Age';
               '__Female';
               '_Married';
               '__Rids<1';
               '1<Rids<7';
               '7<Rid<15';
               '__Adults';
               'Both_Wrk';
               'Not_Empl';
               '_Mar_Fem';
               rname];

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Linear Model %
    x=[i,x2];
    warning off;
    w=inv(z'*z);
    zx=z'*x;
    xxi=inv(zx'*w*zx);
    beta=xxi*(zx'*w*(z'*y));
    e=y-x*beta;
    w=inv((z.*(e*ones(1,length(z(1,:)),1)))'*(z.*(e*ones(1,length(z(1,:)),1))));
    xxi=inv(zx'*w*zx);
    beta=xxi*(zx'*w*(z'*y));
    e=y-x*beta;
    v=inv(zx'*inv((z.*(e*ones(1,length(z(1,:)),1)))'*(z.*(e*ones(1,length(z(1,:)),1))))*zx);
    se=sqrt(diag(v));
    j=(e'*z)*w*(z'*e);
    dof=length(z(1,:))-length(x(1,:));
    fprintf(out,'Variable    Estimate      St Error\n');
    fprintf(out,'----------------------------------\n');
    for iii=1:length(se)
        fprintf(out,'%s      %f      %f\n',xname(iii,:),beta(iii),se(iii));
    end;
    if dof>0
        fprintf(out,'J-Statistic, dof, pvalue: %f   %f   %f\n',j,dof,1-chi2cdf(j,dof));
    end;
    fprintf(out,'----------------------------------\n\n\n\n');
    
    % Linear Spline %
            xname=['_Slope_1';
               '_Slope_2';
               'Has_Retr';
               '__RetInc';
               '_Primary';
               'Some_Hgh';
               'Hign_Grd';
               'Some_Col';
               '_College';
               '_____Age';
               '__Female';
               '_Married';
               '__Rids<1';
               '1<Rids<7';
               '7<Rid<15';
               '__Adults';
               'Both_Wrk';
               'Not_Empl';
               '_Mar_Fem';
               rname];
    ic=i-knot;
    i1=i<=knot;
    i2=1-i1;
    dx=[ic.*i1,ic.*i2];
    x=[dx,x2];
    zx=z'*x;
    xxi=inv(zx'*w*zx);
    beta=xxi*(zx'*w*(z'*y));
    e=y-x*beta;
    v=inv(zx'*inv((z.*(e*ones(1,length(z(1,:)),1)))'*(z.*(e*ones(1,length(z(1,:)),1))))*zx);
    warning on;
    se=sqrt(diag(v));
    j=(e'*z)*w*(z'*e);
    dof=length(z(1,:))-length(x(1,:));
    
    
    fprintf(out,'Variable    Estimate      St Error\n');
    fprintf(out,'----------------------------------\n');
    for iii=1:length(se)
        fprintf(out,'%s      %f      %f\n',xname(iii,:),beta(iii),se(iii));
    end;
    if dof>0
        fprintf(out,'J-Statistic, dof, pvalue: %f   %f   %f\n',j,dof,1-chi2cdf(j,dof));
    end;
    fprintf(out,'----------------------------------\n\n\n\n');
    
    mx=mean(x2)';
    tr=dx*beta(1:2)+mx'*beta(3:length(beta(:,1)));
    gdat=[i,tr];
    gdat=sortrows(gdat,1);
    sh2=strcat(sample,' households');
    plot(gdat(:,1),gdat(:,2));
    title(sh2);
    ylabel('Transfers (Pesos)');
    xlabel('Total Income (Pesos)');
    grid on;
    saveas(gcf,strcat('gmmfig_',num2str(smp),'.eps'),'psc2');
    smp=smp+1;
end
fclose(out);
    
    
    
    
    
        
    

    

    
    
    
    
    
        
    
