%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SMOOTH.M

% This is a Matlab program, written by
% Bruce E. Hansen
% Department of Economics
% University of Wisconsin
% bhansen@ssc.wisc.edu
% http:/www.ssc.wisc.edu/~bhansen

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function smooth()
trm = .02;            % Percentile of high-income households deleted from sample
 
per_ = .05;           % Percentage to trim from quantiles for threshold search @
qn   = 100;           % Set qn=0  to search across all thresholds.
                      % Set qn to a positive integers to search
                      % across a fixed number of quantiles     @
autop_ = 0;           % If set to 1, will automatically print graphs   @
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global i;
global ee;
global xm;
global xx;
global e0;
global ya;
global smp;
global sample;
out=fopen('smooth.txt','wt');
fclose(out);
smp=1;
while smp<=2;  % Set to 1 if "urban" or 2 if "rural"  %
    rname=['Region1_';
        'Region2_';
        'Region3_';
        'Region4_';
        'Region5_';
        'Region6_';
        'Region7_';
        'Region8_';
        'Region9_';
        'Region10';
        'Region11';
        'Region12';
        'Region13'];
    vn=['__Abroad'                     
        'Domestic'                    
        '__Inkind'                     
        '___Gifts'                     
        '__Income'                     
        '___Wages'                     
        '____Self'                     
        '____Rent'                     
        'Interest'                     
        '__RetInc'                     
        'Dividend'                     
        '_Primary'                     
        'Some Hgh'                     
        'High Grd'                     
        'Some Col'                     
        '_College'                     
        '_____Age'                     
        '__Female'                     
        '_Married'                     
        '__Kids<1'                     
        '1<Kids<7'                     
        '7<Kid<15'                     
        '_HH Size'                     
        'Both Wrk'                     
        'Not Empl'                     
        '_Mar Fem'                     
        '__Region'];
    if smp==1;
        sample='Urban';
        rname=['Region0_';rname];
        knot=21840;
        load urban.txt;
        dat=urban;
    elseif smp==2;
        sample='Rural';
        knot=10793;
        load rural.txt;
        dat=rural;
    end;
    
    abroad=dat(:,1);
    domestic=dat(:,2);
    inkind=dat(:,3);
    gifts=dat(:,4);
    totinc=dat(:,5);
    retinc=dat(:,10);
    yname='Transfers';
    
    y = abroad+domestic+inkind-gifts;
    i = totinc+gifts-abroad-domestic-inkind-retinc;
    
    ind=0;
    ttempdat=[dat,i,y];
    for iii=1:length(i);
        if i(iii)>=0
            if ind==0
                tempdat=ttempdat(iii,:);
                ind=1;
            else
                tempdat=[tempdat;ttempdat(iii,:)];
            end;
        end;
    end;
    dat=tempdat;
    dat=sortrows(dat,28);
    n=floor(length(dat(:,1))*(1-trm));
    dat=dat(1:n,:);

    retinc=dat(:,10);
    i=dat(:,28);
    y=dat(:,29);
    dat=dat(:,1:27);
    
    dat(:,23)=dat(:,23)-dat(:,20)-dat(:,21)-dat(:,22);
    xx=[(retinc>0),retinc,dat(:,12:17)];
    xn=['Has_Retr';vn(10,:);vn(12:17,:)];
    region=dat(:,27);
    r=unique(region);
    xreg=(region*ones(1,length(r))==(ones(length(region),1)*r'));%???????%
    xx=[xx,dat(:,18:26),xreg];
    xn=[xn;vn(18:26,:);rname];
    mx=ones(n,1)*mean(xx);
    i0=(i==0);
    xx=[i0,xx];
    xn=['NoIncome';xn];
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    out=fopen('smooth.txt','at');
    fprintf(out,'*********************************************\n');
    fprintf(out,strcat('\n\nDependent Variable: ',yname,'\n'));
    fprintf(out,strcat(sample,' households\n'));
    fprintf(out,'Trimmed Top Quantile of Income:    %f\n',trm);
    fprintf(out,'Number of Zero Income:             %u\n',sum(i0));
    fprintf(out,'Observations:                      %u\n',n);
    fprintf(out,'----------------------------------------------\n');
    fclose(out);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    xm=xx*inv(xx'*xx);
    e0=y-xm*(xx'*y);
    ee0=e0'*e0;
    if qn==0
        q=sortrows(i,1);
        q=unique(q(floor(per_*n):n-floor(per_*n)));
        qn=length(q(:,1));
    else
        q=sortrows(i,1);
        q=q(floor((per_:(1-per_*2)/qn:(qn-1)*(1-per_*2)/qn+per_)*n))';
    end;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Continuous Linear Spline Model (for starting values) %
    sse=zeros(qn,1);
    j=1;
    while j<=qn
        qj=q(j);
        ic=i-qj;
        i1=(i<=qj);
        i2=1-i1;
        dx=[(ic.*i1),(ic.*i2)];
        xv=dx-xm*(xx'*dx);
        et=e0-xv*(e0'/xv')';
        sse(j)=et'*et;
        %if round(j/10)==(j/10)
        %    [j,qj,sse(j)/n]
        %end;
        j=j+1;
    end;
    [temp,jmin]=min(sse);
    qhat=q(jmin);
    
    ic=i-qhat;
    i1=(i<=qhat);
    i2=1-i1;
    dx=[(ic.*i1),(ic.*i2)];
    x=[dx,xx];
    beta=inv(x'*x)*(x'*y);
    e=y-x*beta;
    ee=e'*e;
    ba=beta(3:length(beta(:,1)));
    
    ya=y-x*beta;
    ee=e'*e;
    tr1=dx*beta([1,2])+mx*beta(4:length(beta(:,1)));
    
    theta0=[[qhat;1000],[(qhat-2000);1000],[qhat;5000],[(qhat-2000);8000]];
    tn=length(theta0(:,1));
    theta=zeros(2,tn);
    smin=zeros(tn,1);
    
    opalgr_=5;
    ti=1;
    while ti<=tn
        [theta(:,ti),smin(:,ti)]=fminunc(@ss,theta0(:,ti));
        ti=ti+1;
    end;
    [temp,minind]=min(smin);
    theta1=theta(:,minind);
    
    [theta,smin]=fminunc(@s_smooth,theta0);
    knot=theta(1);
    omega=abs(theta(2));
    ik=i-knot;
    a=ik./omega;
    cdfa=normcdf(a);
    pdfa=normpdf(a);
    dx=[ik.*(1-cdfa),ik.*cdfa]+(pdfa*[(-1),1])*omega;
    x=[dx,xx];
    beta=inv(x'*x)*(x'*y);
    e=y-x*beta;
    ee=e'*e;
    k=length(x(1,:));
    sig=(e'*e)/(n-k-2);
    r_2=1-ee/(sum((y-mean(y)').^2));
    theta=[knot;omega;beta];
    if omega>10
        b1=beta(1);
        b2=beta(2);
        b12=b1-b2;
        g1=-((1-cdfa)*b1+cdfa*b2)/omega+a.*pdfa*b12*(1-omega)/omega;
        g2=-((1-cdfa)*b1+cdfa*b2).*a/omega-pdfa*b12+pdfa.*(a.^2).*(b12*(1-omega)/omega);
        gx=[g1,g2,x];
        h=inv(gx'*gx);
        se=sqrt(diag(h)*sig);
        seh=sqrt(diag(h*(gx.*(e*ones(1,length(gx(1,:)))))'*(gx.*(e*ones(1,length(gx(1,:)))))*h));
    else
        se=zeros(length(theta(:,1)),1);
        seh=se;
    end;
    our=fopen('smooth.txt','at');
    xname =['____Knot';'___Omega';'_Slope_1';'_Slope_2';xn];
    fprintf(out,'Smooth Spline Model\n');
    fprintf(out,'Sum of Squared Errors:       %f\n',ee)
    fprintf(out,'Residual Standard Deviation: %f\n',sqrt(sig));
    fprintf(out,'R-squared:                   %f\n\n',r_2);
    fprintf(out,'Variable    Estimate      St Error     Het St Er\n');
    fprintf(out,'----------------------------------\n');
    for iii=1:length(se)
        fprintf(out,'%s   %f   %f   %f\n',xname(iii,:),theta(iii),se(iii),seh(iii));
    end;
    fclose(out);
    
    mx=ones(n,1)*(mean(xx));
    xr=[dx,mx];
    tr3=xr*beta;
    gr_tr(i,[tr1,tr3]);
    smp=smp+1;    
end;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function s=ss(theta);
global i;
global ee;
global xm;
global xx;
global e0;
global ya;
ik=i-theta(1);
omega=abs(theta(2));
a=ik/omega;
cdfa=normcdf(a);
dx=([ik.*(1-cdfa),ik.*cdfa])+[-normpdf(a),normpdf(a)]*omega;
xv=dx-xm*(xx'*dx);
e=ya-dx*(inv(dx'*dx)*(dx'*ya));
s=(e'*e-ee)/length(e(:,1));
    
function s=s_smooth(theta);
global i;
global ee;
global xm;
global xx;
global e0;
ik=i-theta(1);
omega=abs(theta(2));
a=ik/omega;
cdfa=normcdf(a);
dx=([ik.*(1-cdfa),ik.*cdfa])+[-normpdf(a),normpdf(a)]*omega;
xv=dx-xm*(xx'*dx);
e=e0-xv*(e0'/xv')';
s=(e'*e-ee)/length(e(:,1));
    
function gr_tr(i,tr);
global sample;
global smp;
plot(i,tr(:,1),i,tr(:,2));
if smp==1
    xlim([0,200000]);
    ylim([0,20000]);
elseif smp==2
    xlim([0,80000]);
    ylim([0,10000]);
end;
sh2=strcat(sample,' households');
title(sh2);
xlabel('Total Income (Pesos)');
ylabel('Transfers (Pesos)');
saveas(gcf,strcat('smooth',num2str(smp),'.eps'),'psc2');
grid on;
    
   
    
    
    