%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TEST.M

% This is a MATLAB program, written by
% Bruce E. Hansen, Department of Economics, University of Wisconsin
% bhansen@ssc.wisc.edu
% http:/www.ssc.wisc.edu/~bhansen

% This program tests the spline transfer function model presented in
% "Are Private Transfers Altruistically Motivated?",
% by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.

% This program tests the spline model against polynomial alternatives.

% The results create Table 4 from the paper.

function test()
global qn;
trm = .02;     % Percentile of high-income households trimmed from sample@

qn   = 1;    % number of quantiles used for test. We set qn=100 @
rep  = 1000;   % Number of bootstrap replications.  @
noinc_=1;      % If set to 1, includes dummy for no income @


out=fopen('test.txt','wt');
fclose(out);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

smp=1;
while smp<=2
    rname=['Region1_';
        'Region2_';
        'Region3_';
        'Region4_';
        'Region5_';
        'Region6_';
        'Region7_';
        'Region8_';
        'Region9_';
        'Region10';
        'Region11';
        'Region12';
        'Region13'];
    if smp==1
        rname=['Region0_';rname];
        sample='Urban';
        load urban.txt;
        dat=urban;
    elseif smp==2
        sample='Rural';
        load rural.txt;
        dat=rural;
    end;
    
    out=fopen('test.txt','at');
    fprintf(out,strcat(sample,' households\n'));
    fprintf(out,'Trimmed Top Quantile of Income:   %f\n',trm);
    fprintf(out,'Bootstrap Replications:           %n\n',rep);
    fprintf(out,'Number of Quantiles:              %n\n',qn);
    fprintf(out,'Tests for a Single Knot\n\n');
    fprintf(out,'P       F-test      P-Value\n');
    fclose(out);

    abroad=dat(:,1);
    domestic=dat(:,2);
    inkind=dat(:,3);
    gifts=dat(:,4);
    totinc=dat(:,5);
    retinc=dat(:,10);
    yname='Transfers';
    
    y = abroad+domestic+inkind-gifts;
    i = totinc+gifts-abroad-domestic-inkind-retinc;
    ind=0;
    ttempdat=[dat,i,y];
    for iii=1:length(i);
        if i(iii)>=0
            if ind==0
                tempdat=ttempdat(iii,:);
                ind=1;
            else
                tempdat=[tempdat;ttempdat(iii,:)];
            end;
        end;
    end;
    dat=tempdat;
    
    dat=sortrows(dat,28);
    dat(1:10,28)
    n=floor(length(dat(:,1))*(1-trm));
    dat=dat(1:n,:);
    retinc=dat(:,10);
    i=dat(:,28);
    y=dat(:,29);
    dat=dat(:,1:27);
    
    dat(:,23)=dat(:,23)-dat(:,20)-dat(:,21)-dat(:,22);
    xx=[(retinc>0),retinc,dat(:,12:17)];
    region=dat(:,27);
    r=unique(region);
    xreg=(region*ones(1,length(r))==(ones(length(region),1)*r'));
    
    xx=[xx,dat(:,18:26),xreg];
    mx=ones(n,1)*mean(xx);
    if noinc_==1
        i0=(i==0);
        xx=[i0,xx];
    end;
    
    p=1;
    while p<=7
        % if p<=4;  per_=.05;
        % elseif p<=6; _per=.15;
        % else; per_=.3;
        % endif;
        per_=.05;
        q=sortrows(i,1);
        q=q(floor((per_:(1-per_*2)/qn:(qn-1)*(1-per_*2)/qn+per_)*n));       
        [lr,e]=ttest(y,xx,i,q,p);
        % Bootstrap %
       boot=zeros(rep,1);
       r=1;
       while r<=rep
            yb=e(ceil(unifrnd(0,1,n,1)*n));
            [lrb,eb]=ttest(yb,xx,i,q,p);
            boot(r)=lrb;
            r=r+1;
        end;
        
        pv=mean(boot>=lr)';
        out=fopen('test.txt','at');
        fprintf(out,'%f   %f   %f\n',p,lr,pv);
        fclose(out);
       p=p+1;
    end;
    out=fopen('test.txt','at');
    fprintf(out,'\n\n\n');
    fclose(out);
    smp=smp+1;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [lr,et]=ttest(y,xx,i,q,p);
global qn;
sp=(1:p);
x=[(i*ones(1,p)).^(ones(length(i),1)*sp),xx];
warning off;
xm=x*inv(x'*x);
warning on;
e=y-xm*(x'*y);
ee0=e'*e;
sse=zeros(qn,1);
j=1;
while j<=qn
    knot=q(j);
    dx=(((i-knot)*ones(1,p)).^(ones(length(i),1)*sp)).*((i<=knot)*ones(1,p));
    xv=dx-xm*(x'*dx);
    et=e-xv*div(e,xv);
    sse(j)=et'*et;
    j=j+1;
end;
[temp,j]=min(sse);
knot=q(j);
ssemin = sse(j);
lr=(ee0-ssemin)/ssemin*length(y(:,1));
dx=(((i-knot)*ones(1,p)).^(ones(length(i),1)*sp)).*((i<=knot)*ones(1,p));
xv=dx-xm*(x'*dx);
et=e-xv*div(e,xv);

function beta=div(e,x)
warning off;
lastwarn(' ');
beta=inv(x'*x)*(x'*e);
[mw,idw] = lastwarn;
lastwarn(' ');
warning on;
if (1-(mw==' '))
    beta=pinv(x'*x)*(x'*e);    
end;
if length(beta(1,:))>1
    beta=beta(:,1);
end;


    
    
    
        
        

        
