%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TEST2.M

% This is a Matlab program, written by
% Bruce E. Hansen, Department of Economics, University of Wisconsin
% bhansen@ssc.wisc.edu
% http:/www.ssc.wisc.edu/~bhansen

% This program tests the single-knot specification against a two-knot
% alternative.

% This results generate Table 5 from the paper.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function test2();
global qn;
global per_;
global n;
trm = .02;            % Percentile of high-income households deleted  @
rep  = 1;             % Number of bootstrap replications.  @

per_ = .05;           % Percentage to trim from quantiles for threshold search @
qn   = 10;          %  Set qn to a positive integers to search
                      %   across a fixed number of quantiles     @
noinc_=1;             % If set to 1, includes dummy for no income, 0 @

out=fopen('test2.txt','wt');
fclose(out);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
smp=1;
while smp<=2
    rname=['Region1_';
        'Region2_';
        'Region3_';
        'Region4_';
        'Region5_';
        'Region6_';
        'Region7_';
        'Region8_';
        'Region9_';
        'Region10';
        'Region11';
        'Region12';
        'Region13'];
    if smp==1
        rname=['Region0_';rname];
        sample='Urban';
        load urban.txt;
        dat=urban;
    elseif smp==2
        sample='Rural';
        load rural.txt;
        dat=rural;
    end;
    
    abroad=dat(:,1);
    domestic=dat(:,2);
    inkind=dat(:,3);
    gifts=dat(:,4);
    totinc=dat(:,5);
    retinc=dat(:,10);
    yname='Transfers';
    
    y = abroad+domestic+inkind-gifts;
    i = totinc+gifts-abroad-domestic-inkind-retinc;
    ind=0;
    ttempdat=[dat,i,y];
    for iii=1:length(i);
        if i(iii)>=0
            if ind==0
                tempdat=ttempdat(iii,:);
                ind=1;
            else
                tempdat=[tempdat;ttempdat(iii,:)];
            end;
        end;
    end;
    dat=tempdat;
    
    dat=sortrows(dat,28);
    dat(1:10,28)
    n=floor(length(dat(:,1))*(1-trm));
    dat=dat(1:n,:);
    retinc=dat(:,10);
    i=dat(:,28);
    y=dat(:,29);
    dat=dat(:,1:27);
    
    dat(:,23)=dat(:,23)-dat(:,20)-dat(:,21)-dat(:,22);
    xx=[(retinc>0),retinc,dat(:,12:17)];
    region=dat(:,27);
    r=unique(region);
    xreg=(region*ones(1,length(r))==(ones(length(region),1)*r'));
    
    xx=[xx,dat(:,18:26),xreg];
    mx=ones(n,1)*mean(xx);
    if noinc_==1
        i0=(i==0);
        xx=[i0,xx];
    end;
    
    clear dat;
    q=i(floor((per_:(1-per_*2)/qn:(qn-1)*(1-per_*2)/qn+per_)*n));
    [f,yp,e] = ttest2(y,xx,i,q);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Bootstrap %
    boot=zeros(rep,1);
    r=1;
    while r<=rep
        yb=e(ceil(unifrnd(0,1,n,1)*n));
        [fb,ypb,eb]=ttest2(yb,xx,i,q);
        boot(r)=fb;
        r=r+1;
    end;
    pv=mean(boot>=f);
    out=fopen('test2.txt','at');
    fprintf(out,strcat(sample,' households\n'));
    fprintf(out,'Trimmed Top Quantile of Income:   %f\n',trm);
    fprintf(out,'Bootstrap Replications:           %n\n',rep);
    fprintf(out,'Number of Quantiles:              %n\n',qn);
    fprintf(out,'F Tests for Second Knot\n\n');
    fprintf(out,'    F-test      P-Value\n');
    fprintf(out,'    %f   %f\n',f,pv);
    fclose(out);
    smp=smp+1;
end;
function [f,yp,et]=ttest2(y,xx,i,q);
global qn;
global per_;
global n;
xm=xx*inv(xx'*xx);
e=y-xm*(xx'*y);
ee0=e'*e;
sse=zeros(qn,1);
j1=1;
while j1<=qn
    k1=q(j1);
    i1=(i<k1);
    i2=1-i1;
    x1=(i-k1).*i1;
    x2=(i-k1).*i2;
    dx=[x1,x2];
    xv=dx-xm*(xx'*dx);
    et=e-xv*div(e,xv);
    sse(j1)=et'*et;
    j1=j1+1;
end;
[sse0,j1]=min(sse);
k1=q(j1);

i1=(i<k1);
x1=i.*i1+(1-i1)*k1;
sse=zeros(qn,1);
j2=1;
while j2<=qn
    if abs(j1-j2)>=ceil(per_*qn)
        k2=q(j2);
        i2=(i<k2)-i1;
        i3=1-i1-i2;
        x2=(i-k1).*i2+i3*(k2-k1);
        x3=(i-k2).*i3;
        dx=[x1,x2,x3];
        xv=dx-xm*(xx'*dx);
        et=e-xv*div(e,xv);
        sse(j2)=et'*et-sse0;
    end;
    j2=j2+1;
end;
[sse1,j2]=min(sse);
k2=q(j2);
sse1=sse1+sse0;
f=(sse0-sse1)/sse1*n;
i2=(i<k2)-i1;
i3=1-i1-i2;
x2=(i-k1).*i2+i3*(k2-k1);
x3=(i-k2).*i3;
dx=[x1,x2,x3];
xv=dx-xm*(xx'*dx);
beta=div(e,xv);
et=e-xv*beta;
yp=dx*beta;

function beta=div(e,x)
warning off;
lastwarn(' ');
beta=inv(x'*x)*(x'*e);
[mw,idw] = lastwarn;
lastwarn(' ');
warning on;
if (1-(mw==' '))
    beta=pinv(x'*x)*(x'*e);    
end;
if length(beta(1,:))>1
    beta=beta(:,1);
end;


